@@ -1,22 +1,22 @@
-#:#
-#:#  *Trial design classes*
-#:# 
-#:#  This file is part of the R package rpact: 
-#:#  Confirmatory Adaptive Clinical Trial Design and Analysis
-#:# 
-#:#  Author: Gernot Wassmer, PhD, and Friedrich Pahlke, PhD
-#:#  Licensed under "GNU Lesser General Public License" version 3
-#:#  License text can be found here: https://www.r-project.org/Licenses/LGPL-3
-#:# 
-#:#  RPACT company website: https://www.rpact.com
-#:#  rpact package website: https://www.rpact.org
-#:# 
-#:#  Contact us for information about our services: info@rpact.com
-#:# 
-#:#  File version: $Revision: 3821 $
-#:#  Last changed: $Date: 2020-11-03 08:59:30 +0100 (Tue, 03 Nov 2020) $
-#:#  Last changed by: $Author: pahlke $
-#:# 
+## |
+## |  *Trial design classes*
+## | 
+## |  This file is part of the R package rpact: 
+## |  Confirmatory Adaptive Clinical Trial Design and Analysis
+## | 
+## |  Author: Gernot Wassmer, PhD, and Friedrich Pahlke, PhD
+## |  Licensed under "GNU Lesser General Public License" version 3
+## |  License text can be found here: https://www.r-project.org/Licenses/LGPL-3
+## | 
+## |  RPACT company website: https://www.rpact.com
+## |  rpact package website: https://www.rpact.org
+## | 
+## |  Contact us for information about our services: info@rpact.com
+## | 
+## |  File version: $Revision: 5577 $
+## |  Last changed: $Date: 2021-11-19 09:14:42 +0100 (Fr, 19 Nov 2021) $
+## |  Last changed by: $Author: pahlke $
+## | 
 
 
 #' @include f_core_constants.R
@@ -53,6 +53,7 @@ TrialDesign <- setRefClass("TrialDesign",
 	contains = "ParameterSet",
 	
 	fields = list(
+		.plotSettings = "PlotSettings",
 		kMax = "integer", 
 		alpha = "numeric",
 		stages = "integer",
@@ -86,6 +87,8 @@ TrialDesign <- setRefClass("TrialDesign",
 				bindingFutility = bindingFutility,
 				tolerance = tolerance)
 			
+			.plotSettings <<- PlotSettings()
+			
 			if (inherits(.self, "TrialDesignConditionalDunnett")) {
 				.parameterNames <<- C_PARAMETER_NAMES
 			} else {
@@ -230,7 +233,7 @@ TrialDesignCharacteristics <- setRefClass("TrialDesignCharacteristics",
 	methods = list(
 		initialize = function(design, ...) {
 			callSuper(.design = design, ...)
-			.parameterNames <<- .getParameterNames(design)
+			.parameterNames <<- .getParameterNames(design = design)
 			.parameterFormatFunctions <<- C_PARAMETER_FORMAT_FUNCTIONS
 			.initStages()
 		},
@@ -264,10 +267,6 @@ TrialDesignCharacteristics <- setRefClass("TrialDesignCharacteristics",
 			}
 		},
 		
-		.getUserDefinedParameters = function() {
-			return("design")
-		},
-		
 		.toString = function(startWithUpperCase = FALSE) {
 			return(paste(.design$.toString(startWithUpperCase = startWithUpperCase), "characteristics"))
 		}
@@ -438,7 +437,7 @@ TrialDesignFisher <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_FISHER,
 	)
 )
 
-#'					
+#'
 #' @name TrialDesignInverseNormal
 #' 
 #' @title
@@ -468,6 +467,8 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 		typeOfDesign = "character",
 		beta = "numeric", 
 		deltaWT = "numeric",
+		deltaPT1 = "numeric",
+		deltaPT0 = "numeric",
 		futilityBounds = "numeric",
 		gammaA = "numeric",
 		gammaB = "numeric",
@@ -489,6 +490,8 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 			futilityBounds = NA_real_,
 			typeOfDesign = C_DEFAULT_TYPE_OF_DESIGN,
 			deltaWT = NA_real_,
+			deltaPT1 = NA_real_,
+			deltaPT0 = NA_real_,
 			optimizationCriterion = C_OPTIMIZATION_CRITERION_DEFAULT,
 			gammaA = NA_real_,
 			gammaB = NA_real_,
@@ -505,6 +508,8 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 				futilityBounds = futilityBounds, 
 				typeOfDesign = typeOfDesign, 
 				deltaWT = deltaWT,
+				deltaPT1 = deltaPT1,
+				deltaPT0 = deltaPT0,
 				optimizationCriterion = optimizationCriterion,
 				gammaA = gammaA,
 				gammaB = gammaB,
@@ -523,6 +528,8 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 				"futilityBounds",
 				"typeOfDesign",
 				"deltaWT",
+				"deltaPT1",
+				"deltaPT0",
 				"optimizationCriterion",
 				"gammaA",
 				"gammaB",
@@ -538,18 +545,31 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 			.initParameterTypes()
 			.initStages()
 		},
-		
-		.pasteComparisonResult = function(name, newValue, oldValue) {
-			return(paste0(name, "_new = ", .arrayToString(newValue), " (", class(newValue), "), ", 
-					name, "_old = ", .arrayToString(oldValue), " (", class(oldValue), ")"))
+
+		.formatComparisonResult = function(x) {
+			if (is.null(x) || length(x) == 0 || !is.numeric(x)) {
+				return(x)
+			}
+			
+			s <- sprintf("%.9f", x)
+			s <- sub("\\.0+", "", s)
+			return(s)
 		},
 		
-		hasChanged = function(kMax, 
+		.pasteComparisonResult = function(name, newValue, oldValue) {
+			return(paste0(name, "_new = ", .arrayToString(.formatComparisonResult(newValue)), " (", class(newValue), "), ", 
+					name, "_old = ", .arrayToString(.formatComparisonResult(oldValue)), " (", class(oldValue), ")"))
+		},
+		
+		hasChanged = function(...,
+				kMax, 
 				alpha, 
 				beta, 
 				sided, 
 				typeOfDesign, 
 				deltaWT, 
+				deltaPT1, 
+				deltaPT0, 
 				informationRates, 
 				futilityBounds, 
 				optimizationCriterion, 
@@ -575,49 +595,69 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 			if (!identical(alpha, .self$alpha)) return(.pasteComparisonResult("alpha", alpha, .self$alpha))
 			if (!identical(beta, .self$beta)) return(.pasteComparisonResult("beta", beta, .self$beta))
 			if (!identical(sided, .self$sided)) return(.pasteComparisonResult("sided", sided, .self$sided))
-			if (kMax > 1) {
-				if (!identical(typeOfDesign, .self$typeOfDesign)) {
-					return(.pasteComparisonResult("typeOfDesign", typeOfDesign, .self$typeOfDesign))
+			if (!identical(twoSidedPower, .self$twoSidedPower)) {
+				return(.pasteComparisonResult("twoSidedPower", twoSidedPower, .self$twoSidedPower))
+			}
+			if (kMax == 1) {
+				return(FALSE)
+			}
+			
+			if (!identical(typeOfDesign, .self$typeOfDesign)) {
+				return(.pasteComparisonResult("typeOfDesign", typeOfDesign, .self$typeOfDesign))
+			}
+			if (typeOfDesign == C_TYPE_OF_DESIGN_WT) {
+				if (!identical(deltaWT, .self$deltaWT)) {
+					return(.pasteComparisonResult("deltaWT", deltaWT, .self$deltaWT))
 				}
-				if (typeOfDesign == C_TYPE_OF_DESIGN_WT) {
-					if (!identical(deltaWT, .self$deltaWT)) {
-						return(.pasteComparisonResult("deltaWT", deltaWT, .self$deltaWT))
-					}
+			}
+			if (typeOfDesign == C_TYPE_OF_DESIGN_PT) {
+				if (!identical(deltaPT1, .self$deltaPT1)) {
+					return(.pasteComparisonResult("deltaPT1", deltaPT1, .self$deltaPT1))
 				}
-				if (!identical(informationRatesTemp, .self$informationRates)) {
-					return(.pasteComparisonResult("informationRates", informationRatesTemp, .self$informationRates))
+				if (!identical(deltaPT0, .self$deltaPT0)) {
+					return(.pasteComparisonResult("deltaPT0", deltaPT0, .self$deltaPT0))
 				}
-				if (typeBetaSpending == C_TYPE_OF_DESIGN_BS_NONE && !identical(futilityBoundsTemp, .self$futilityBounds)) {
-					return(.pasteComparisonResult("futilityBounds", futilityBoundsTemp, .self$futilityBounds))
-				}
-				if (!identical(optimizationCriterion, .self$optimizationCriterion)) {
-					return(.pasteComparisonResult("optimizationCriterion", optimizationCriterion, .self$optimizationCriterion))
-				}
-				if (!identical(typeBetaSpending, .self$typeBetaSpending)) {
-					return(.pasteComparisonResult("typeBetaSpending", typeBetaSpending, .self$typeBetaSpending))
-				}
-				if (!identical(gammaA, .self$gammaA)) {
-					return(.pasteComparisonResult("gammaA", gammaA, .self$gammaA))
-				}
-				if (!identical(gammaB, .self$gammaB)) {
-					return(.pasteComparisonResult("gammaB", gammaB, .self$gammaB))
-				}
-				if (any(na.omit(futilityBounds) > -6) && !identical(bindingFutility, .self$bindingFutility)) {
-					return(.pasteComparisonResult("bindingFutility", bindingFutility, .self$bindingFutility))
-				}
-				if (!identical(userAlphaSpending, .self$userAlphaSpending)) {
-					return(.pasteComparisonResult("userAlphaSpending", userAlphaSpending, .self$userAlphaSpending))
-				}
-				if (!identical(userBetaSpending, .self$userBetaSpending)) {
-					return(.pasteComparisonResult("userBetaSpending", userBetaSpending, .self$userBetaSpending))
-				}
-				if (!identical(twoSidedPower, .self$twoSidedPower)) {
-					return(.pasteComparisonResult("twoSidedPower", twoSidedPower, .self$twoSidedPower))
-				}
-				if (typeOfDesign == C_TYPE_OF_DESIGN_HP) {
-					if (!identical(constantBoundsHP, .self$constantBoundsHP)) {
-						return(.pasteComparisonResult("constantBoundsHP", constantBoundsHP, .self$constantBoundsHP))
-					}
+			}
+			if (!identical(informationRatesTemp, .self$informationRates)) {
+				return(.pasteComparisonResult("informationRates", informationRatesTemp, .self$informationRates))
+			}
+			if (.getParameterType("futilityBounds") != C_PARAM_GENERATED &&
+					(!grepl("^as.*", typeOfDesign) || typeBetaSpending == C_TYPE_OF_DESIGN_BS_NONE) && 
+					!identical(futilityBoundsTemp, .self$futilityBounds)) {
+				return(.pasteComparisonResult("futilityBounds", futilityBoundsTemp, .self$futilityBounds))
+			}
+			if (!identical(optimizationCriterion, .self$optimizationCriterion)) {
+				return(.pasteComparisonResult("optimizationCriterion", optimizationCriterion, .self$optimizationCriterion))
+			}
+			if (!identical(typeBetaSpending, .self$typeBetaSpending)) {
+				return(.pasteComparisonResult("typeBetaSpending", typeBetaSpending, .self$typeBetaSpending))
+			}
+			if (!identical(gammaA, .self$gammaA)) {
+				return(.pasteComparisonResult("gammaA", gammaA, .self$gammaA))
+			}
+			if (!identical(gammaB, .self$gammaB)) {
+				return(.pasteComparisonResult("gammaB", gammaB, .self$gammaB))
+			}
+			if ((typeOfDesign == C_TYPE_OF_DESIGN_PT && !identical(bindingFutility, .self$bindingFutility)) || 
+					(!identical(bindingFutility, .self$bindingFutility) &&
+					.getParameterType("futilityBounds") != C_PARAM_NOT_APPLICABLE &&
+					(sided == 1 || !grepl("^as.*", typeOfDesign) || typeBetaSpending == C_TYPE_OF_DESIGN_BS_NONE) &&
+					(any(na.omit(futilityBounds) > -6) || any(na.omit(.self$futilityBounds) > -6)) 
+					)) {
+				return(.pasteComparisonResult("bindingFutility", bindingFutility, .self$bindingFutility))
+			}
+			if (!identical(userAlphaSpending, .self$userAlphaSpending)) {
+				return(.pasteComparisonResult("userAlphaSpending", userAlphaSpending, .self$userAlphaSpending))
+			}
+			if (!identical(userBetaSpending, .self$userBetaSpending)) {
+				return(.pasteComparisonResult("userBetaSpending", userBetaSpending, .self$userBetaSpending))
+			}
+			if (!identical(twoSidedPower, .self$twoSidedPower)) {
+				return(.pasteComparisonResult("twoSidedPower", twoSidedPower, .self$twoSidedPower))
+			}
+			if (typeOfDesign == C_TYPE_OF_DESIGN_HP) {
+				if (!identical(constantBoundsHP, .self$constantBoundsHP)) {
+					return(.pasteComparisonResult("constantBoundsHP", constantBoundsHP, .self$constantBoundsHP))
 				}
 			}
 			return(FALSE)
@@ -635,6 +675,8 @@ TrialDesignInverseNormal <- setRefClass(C_CLASS_NAME_TRIAL_DESIGN_INVERSE_NORMAL
 				"power",
 				"twoSidedPower",
 				"deltaWT",
+				"deltaPT1",
+				"deltaPT0",
 				"futilityBounds",
 				"bindingFutility",
 				"constantBoundsHP",
@@ -775,15 +817,15 @@ TrialDesignConditionalDunnett <- setRefClass(
 #' @inheritParams param_alpha
 #' @param informationAtInterim The information to be expected at interim, default is \code{informationAtInterim = 0.5}.   
 #' @param secondStageConditioning The way the second stage p-values are calculated within the closed system of hypotheses.
-#' 		If \code{secondStageConditioning = FALSE} is specified, the unconditional adjusted p-values are used, otherwise 
-#'  	conditional adjusted p-values are calculated, default is \code{secondStageConditioning = TRUE} 
-#'      (for details, see König et al., 2008).
+#'        If \code{secondStageConditioning = FALSE} is specified, the unconditional adjusted p-values are used, otherwise 
+#'  	  conditional adjusted p-values are calculated, default is \code{secondStageConditioning = TRUE} 
+#'        (for details, see Koenig et al., 2008).
 #'
 #' @details 
 #' For performing the conditional Dunnett test the design must be defined through this function. 
 #' You can define the information fraction and the way of how to compute the second stage 
 #' p-values only in the design definition, and not in the analysis call.\cr 
-#' See \code{\link{getClosedConditionalDunnettTestResults}} for an example and König et al. (2008) and
+#' See \code{\link{getClosedConditionalDunnettTestResults}} for an example and Koenig et al. (2008) and
 #' Wassmer & Brannath (2016), chapter 11 for details of the test procedure.
 #' 
 #' @template return_object_trial_design
@@ -825,6 +867,7 @@ getDesignConditionalDunnett <- function(alpha = 0.025, # C_ALPHA_DEFAULT
 #' @inheritParams param_nMax
 #' @inheritParams param_plotPointsEnabled
 #' @inheritParams param_showSource
+#' @inheritParams param_plotSettings
 #' @inheritParams param_legendPosition
 #' @inheritParams param_grid
 #' @param type The plot type (default = \code{1}). The following plot types are available:
@@ -847,7 +890,7 @@ getDesignConditionalDunnett <- function(alpha = 0.025, # C_ALPHA_DEFAULT
 #' Note that \code{\link[=param_nMax]{nMax}} is not an argument that it passed to \code{ggplot2}. 
 #' Rather, the underlying calculations (e.g. power for different theta's or average sample size) are based 
 #' on calls to function \code{\link{getPowerAndAverageSampleNumber}} which has argument \code{\link[=param_nMax]{nMax}}. 
-#' I.e. \code{\link[=param_nMax]{nMax}} is not an argument to ggplot2 but to \code{\link{getPowerAndAverageSampleNumber}} 
+#' I.e., \code{\link[=param_nMax]{nMax}} is not an argument to ggplot2 but to \code{\link{getPowerAndAverageSampleNumber}} 
 #' which is called prior to plotting.
 #' 
 #' @seealso \code{\link{plot.TrialDesignSet}} to compare different designs or design parameters visual.
@@ -867,23 +910,28 @@ getDesignConditionalDunnett <- function(alpha = 0.025, # C_ALPHA_DEFAULT
 #' 
 #' @export
 #' 
-plot.TrialDesign = function(x, y, ..., main = NA_character_,
+plot.TrialDesign <- function(x, y, ..., main = NA_character_,
 		xlab = NA_character_, ylab = NA_character_, type = 1L, palette = "Set1",
 		theta = seq(-1, 1, 0.01), nMax = NA_integer_, plotPointsEnabled = NA, 
 		legendPosition = NA_integer_, showSource = FALSE, 
-		grid = 1) {
+		grid = 1, plotSettings = NULL) {
 		
 	fCall = match.call(expand.dots = FALSE)
 	designName <- deparse(fCall$x)
+	.assertIsSingleInteger(grid, "grid", validateType = FALSE)
 	typeNumbers <- .getPlotTypeNumber(type, x)
+	if (is.null(plotSettings)) {
+		plotSettings <- .getGridPlotSettings(x, typeNumbers, grid)
+	}
 	p <- NULL
 	plotList <- list()
 	for (typeNumber in typeNumbers) {
 		p <- .plotTrialDesign(x = x, y = y, main = main,
 			xlab = xlab, ylab = ylab, type = typeNumber, palette = palette,
 			theta = theta, nMax = nMax, plotPointsEnabled = plotPointsEnabled, 
-			legendPosition = legendPosition, showSource = showSource, 
-			designName = designName, ...)
+			legendPosition = .getGridLegendPosition(legendPosition, typeNumbers, grid), 
+			showSource = showSource, designName = designName, 
+			plotSettings = plotSettings, ...)
 		.printPlotShowSourceSeparator(showSource, typeNumber, typeNumbers)
 		if (length(typeNumbers) > 1) {
 			caption <- .getPlotCaption(x, typeNumber, stopIfNotFound = TRUE)
@@ -905,10 +953,10 @@ plot.TrialDesign = function(x, y, ..., main = NA_character_,
 	return(.createPlotResultObject(plotList, grid))
 }
 
-.plotTrialDesign = function(..., x, y, main,
+.plotTrialDesign <- function(..., x, y, main,
 		xlab, ylab, type, palette,
 		theta, nMax, plotPointsEnabled, 
-		legendPosition, showSource, designName) {
+		legendPosition, showSource, designName, plotSettings = NULL) {
 		
 	.assertGgplotIsInstalled()
 	
@@ -941,6 +989,9 @@ plot.TrialDesign = function(x, y, ..., main = NA_character_,
 		designSet <- getDesignSet(designs = c(x, y), variedParameters = variedParameters)
 	} else {
 		designSet <- TrialDesignSet(design = x, singleDesign = TRUE)
+		if (!is.null(plotSettings)) {
+			designSet$.plotSettings <- plotSettings
+		}
 	}
 	
 	.plotTrialDesignSet(x = designSet, y = y, main = main, xlab = xlab, ylab = ylab, type = type,
