@@ -1,22 +1,22 @@
-#:#
-#:#  *Core assertions*
-#:# 
-#:#  This file is part of the R package rpact: 
-#:#  Confirmatory Adaptive Clinical Trial Design and Analysis
-#:# 
-#:#  Author: Gernot Wassmer, PhD, and Friedrich Pahlke, PhD
-#:#  Licensed under "GNU Lesser General Public License" version 3
-#:#  License text can be found here: https://www.r-project.org/Licenses/LGPL-3
-#:# 
-#:#  RPACT company website: https://www.rpact.com
-#:#  rpact package website: https://www.rpact.org
-#:# 
-#:#  Contact us for information about our services: info@rpact.com
-#:# 
-#:#  File version: $Revision: 3989 $
-#:#  Last changed: $Date: 2020-11-23 11:25:20 +0100 (Mon, 23 Nov 2020) $
-#:#  Last changed by: $Author: pahlke $
-#:# 
+## |
+## |  *Core assertions*
+## | 
+## |  This file is part of the R package rpact: 
+## |  Confirmatory Adaptive Clinical Trial Design and Analysis
+## | 
+## |  Author: Gernot Wassmer, PhD, and Friedrich Pahlke, PhD
+## |  Licensed under "GNU Lesser General Public License" version 3
+## |  License text can be found here: https://www.r-project.org/Licenses/LGPL-3
+## | 
+## |  RPACT company website: https://www.rpact.com
+## |  rpact package website: https://www.rpact.org
+## | 
+## |  Contact us for information about our services: info@rpact.com
+## | 
+## |  File version: $Revision: 5615 $
+## |  Last changed: $Date: 2021-12-06 09:29:15 +0100 (Mo, 06 Dez 2021) $
+## |  Last changed by: $Author: wassmer $
+## | 
 
 .stopWithWrongDesignMessage <- function(design) {
 	stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'design' must be an instance of ", .arrayToString(
@@ -179,41 +179,36 @@
 }
 
 .isStageResults <- function(stageResults) {
-	return(.isStageResultsMeans(stageResults) || 
-			.isStageResultsRates(stageResults) || 
-			.isStageResultsSurvival(stageResults) ||
-			.isStageResultsMeansMultiArm(stageResults) || 
-			.isStageResultsRatesMultiArm(stageResults) || 
-			.isStageResultsSurvivalMultiArm(stageResults))
+	return(inherits(stageResults, "StageResults"))
 }
 
-.isStageResultsMeans <- function(stageResults) {
-	return(class(stageResults) == "StageResultsMeans")
-}
-
-.isStageResultsMeansMultiArm <- function(stageResults) {
+.isStageResultsMultiArmMeans <- function(stageResults) {
 	return(class(stageResults) == "StageResultsMultiArmMeans")
 }
 
-.isStageResultsRates <- function(stageResults) {
-	return(class(stageResults) == "StageResultsRates")
-}
-
-.isStageResultsRatesMultiArm <- function(stageResults) {
+.isStageResultsMultiArmRates <- function(stageResults) {
 	return(class(stageResults) == "StageResultsMultiArmRates")
 }
-.isStageResultsSurvival <- function(stageResults) {
-	return(class(stageResults) == "StageResultsSurvival")
+
+.isStageResultsMultiArmSurvival <- function(stageResults) {
+	return(class(stageResults) == "StageResultsMultiArmSurvival")
 }
 
-.isStageResultsSurvivalMultiArm <- function(stageResults) {
-	return(class(stageResults) == "StageResultsMultiArmSurvival")
+.isStageResultsEnrichmentMeans <- function(stageResults) {
+	return(class(stageResults) == "StageResultsEnrichmentMeans")
+}
+
+.isStageResultsEnrichmentRates <- function(stageResults) {
+	return(class(stageResults) == "StageResultsEnrichmentRates")
+}
+
+.isStageResultsEnrichmentSurvival <- function(stageResults) {
+	return(class(stageResults) == "StageResultsEnrichmentSurvival")
 }
 
 .assertIsStageResults <- function(stageResults) {
 	if (!.isStageResults(stageResults)) {
-		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'stageResults' must be an instance of ", 
-			.arrayToString(.getStageResultsClassNames(), vectorLookAndFeelEnabled = FALSE), 
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'stageResults' must be a 'StageResults' object", 
 			" (is '", class(stageResults), "')")
 	}
 }
@@ -334,11 +329,11 @@
 		
 		if (.isDatasetSurvival(dataInput) ) {
 			if (any(na.omit(dataInput$getOverallEventsUpTo(stage)) < 0)) {
-				stop(C_EXCEPTION_TYPE_ILLEGAL_DATA_INPUT, "all overall events must be >= 0")
+				stop(C_EXCEPTION_TYPE_ILLEGAL_DATA_INPUT, "all cumulative events must be >= 0")
 			}
 			
 			if (any(na.omit(dataInput$getOverallAllocationRatiosUpTo(stage)) <= 0)) {
-				stop(C_EXCEPTION_TYPE_ILLEGAL_DATA_INPUT, "all overall allocation ratios must be > 0")
+				stop(C_EXCEPTION_TYPE_ILLEGAL_DATA_INPUT, "all cumulative allocation ratios must be > 0")
 			}
 		}
 	}
@@ -351,7 +346,7 @@
 			plural <- ifelse(s == 1, "", "s")
 			warning(sprintf(paste0("The data of the last %s in the dataset will be ",
 					"ignored because the design has specified kMax = %s"), 
-				ifelse(s == 1, "stage", paste0(s, " stages")), kMax, kMax), call. = FALSE)
+				ifelse(s == 1, "stage", paste0(s, " stages")), kMax), call. = FALSE)
 		}
 		else if (numberOfStages < kMax) {
 			dataInput$.fillWithNAs(kMax)
@@ -394,15 +389,15 @@
 }
 
 .isDatasetMeans <- function(dataInput) {
-	return(class(dataInput) == "DatasetMeans")
+	return(inherits(dataInput, "DatasetMeans"))
 }
 
 .isDatasetRates <- function(dataInput) {
-	return(class(dataInput) == "DatasetRates")
+	return(inherits(dataInput, "DatasetRates"))
 }
 
 .isDatasetSurvival <- function(dataInput) {
-	return(class(dataInput) == "DatasetSurvival")
+	return(inherits(dataInput, "DatasetSurvival"))
 }
 
 .assertIsNumericVector <- function(x, argumentName, naAllowed = FALSE, noDefaultAvailable = FALSE) {
@@ -523,12 +518,12 @@
 	}
 	
 	if (!is.numeric(x) || (!naAllowed && is.na(x)) || (validateType && !is.integer(x)) || 
-			(!validateType && !is.na(x) && as.integer(x) != x)) {
+			(!validateType && !is.na(x) && !is.infinite(x) && as.integer(x) != x)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
 			"'", argumentName, "' (", ifelse(isS4(x), class(x), x), ") must be a ", prefix, "integer value")
 	}
 	
-	if (mustBePositive && !is.na(x) && x <= 0) {
+	if (mustBePositive && !is.na(x) && !is.infinite(x) && x <= 0) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
 			"'", argumentName, "' (", ifelse(isS4(x), class(x), x), ") must be a ", prefix, "integer value")
 	}
@@ -755,40 +750,38 @@
 }
 
 .assertIsValidLegendPosition <- function(legendPosition) {
-	if (is.null(legendPosition) || length(legendPosition) == 0 || 
-			(!is.na(legendPosition) && !is.numeric(legendPosition))) {
+	if (is.null(legendPosition) || length(legendPosition) != 1) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
-			"'legendPosition' (", legendPosition, ") must be a valid integer value")
+			"'legendPosition' (", .arrayToString(legendPosition), ") must be a single integer or character value")
 	}
 	
-	if (!is.na(legendPosition) && (legendPosition < -1 || legendPosition > 6)) {
-		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'legendPosition' (", 
-			legendPosition, ") must be an integer between -1 and 6")
+	if (is.na(legendPosition)) {
+		return(invisible())
+	}
+	
+	if (!is.numeric(legendPosition) && !is.character(legendPosition)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
+			"'legendPosition' (", legendPosition, ") must be a single integer or character value")
+	}
+	
+	if (is.numeric(legendPosition)) {
+		.assertIsSingleInteger(legendPosition, "legendPosition", validateType = FALSE)
+		.assertIsInClosedInterval(legendPosition, "legendPosition", lower = -1, upper = 6)
+	} else {
+		validLegendPositions <- c("none", "top", "bottom", "left", "right")
+		if (!(legendPosition %in% validLegendPositions)) {
+			stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
+				"'legendPosition' (", legendPosition, ") must be one of the following values: ",
+				.arrayToString(validLegendPositions))
+		}
 	}
 }
 
-.assertIsValidKMax <- function(kMax, kMaxLowerBound = 1, kMaxUpperBound = C_KMAX_UPPER_BOUND) {
-	if (missing(kMax)) {
-		stop(C_EXCEPTION_TYPE_MISSING_ARGUMENT, "'kMax' must be defined")
-	}
-	
-	if (is.null(kMax) || length(kMax) == 0 || sum(is.na(kMax)) > 0) {
-		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'kMax' is invalid")
-	}
-	
-	if (kMax == Inf || kMax == -Inf) {
-		stop(sprintf(paste0(C_EXCEPTION_TYPE_ARGUMENT_OUT_OF_BOUNDS, 
-					"'kMax' (%s) is out of bounds [%s; %s]"), kMax, kMaxLowerBound, kMaxUpperBound))
-	}
-	
-	if (kMax != as.integer(kMax)) {
-		stop(sprintf(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
-				"'kMax' (%s) must be a valid integer (is an instance of class '%s')", kMax, class(kMax)))
-	}
-	
-	if (kMax < kMaxLowerBound || kMax > kMaxUpperBound) {
-		stop(sprintf(paste0(C_EXCEPTION_TYPE_ARGUMENT_OUT_OF_BOUNDS, 
-				"'kMax' (%s) is out of bounds [%s; %s]"), kMax, kMaxLowerBound, kMaxUpperBound))
+.assertIsValidKMax <- function(kMax, kMaxLowerBound = 1, kMaxUpperBound = C_KMAX_UPPER_BOUND, showWarnings = FALSE) {
+	.assertIsSingleInteger(kMax, "kMax", validateType = FALSE)
+	.assertIsInClosedInterval(kMax, "kMax", lower = kMaxLowerBound, upper = kMaxUpperBound)
+	if (showWarnings && kMax > 10) {
+		warning("The usage of 'kMax' (", kMax, ") > 10 is not validated", call. = FALSE)
 	}
 }
 
@@ -940,7 +933,14 @@
 			") must be equal to 'kMax' (", kMax, ") - 1")
 	}
 	
-	.assertValuesAreInsideBounds("futilityBounds", futilityBounds, -6, 6)
+	.assertValuesAreInsideBounds("futilityBounds", futilityBounds, -Inf, 6)
+	
+}
+
+.assertIsValidCipher <- function(key, value) {
+	if (getCipheredValue(value) != C_CIPHERS[[key]]) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'token' and/or 'secret' unkown")
+	}
 }
 
 .assertIsValidAlpha0Vec <- function(alpha0Vec, kMax = length(alpha0Vec) - 1, 
@@ -1167,7 +1167,7 @@
 		return(FALSE)
 	}
 	
-	return(sum(futilityBounds == C_FUTILITY_BOUNDS_DEFAULT) == 0)
+	return(any(na.omit(futilityBounds) > C_FUTILITY_BOUNDS_DEFAULT))
 }
 
 .isTrialDesignWithValidAlpha0Vec <- function(design) {
@@ -1185,13 +1185,13 @@
 		return(FALSE)
 	}
 	
-	return(sum(alpha0Vec == C_ALPHA_0_VEC_DEFAULT) == 0)
+	return(any(alpha0Vec != C_ALPHA_0_VEC_DEFAULT))
 }
 
 .assertPackageIsInstalled <- function(packageName) {
 	if (!requireNamespace(packageName, quietly = TRUE)) {
 		stop("Package \"", packageName, "\" is needed for this function to work. ", 
-			"Please install and load it", call. = FALSE)
+			"Please install using, e.g., install.packages(\"", packageName, "\")", call. = FALSE)
 	}
 }
 
@@ -1367,20 +1367,29 @@
 }
 
 .assertIsValidAllocationRatioPlanned <- function(allocationRatioPlanned, numberOfGroups) {
+	if (numberOfGroups == 1) {
+		return(invisible())
+	}
+	
 	.assertIsSingleNumber(allocationRatioPlanned, "allocationRatioPlanned")
-	.assertIsInOpenInterval(allocationRatioPlanned, "allocationRatioPlanned", 0, C_ALLOCATION_RATIO_MAXIMUM)
+	.assertIsInOpenInterval(allocationRatioPlanned, 
+		"allocationRatioPlanned", 0, C_ALLOCATION_RATIO_MAXIMUM)
 	if (allocationRatioPlanned != C_ALLOCATION_RATIO_DEFAULT && numberOfGroups == 1) {
 		warning("Planned allocation ratio ", allocationRatioPlanned, " will be ignored ",
-			"because the dataset has only one group", call. = FALSE)
+			"because the specified data has only one group", call. = FALSE)
 	}
 }
 
-.assertIsValidAllocationRatioPlannedSampleSize <- function(allocationRatioPlanned, maxNumberOfSubjects = NA_real_) {
+.assertIsValidAllocationRatioPlannedSampleSize <- function(
+		allocationRatioPlanned, maxNumberOfSubjects = NA_real_) {
+		
 	.assertIsSingleNumber(allocationRatioPlanned, "allocationRatioPlanned")
+	
 	if (allocationRatioPlanned < 0) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
 			"'allocationRatioPlanned' (", allocationRatioPlanned, ") must be >= 0")
 	}
+	
 	if (length(maxNumberOfSubjects) > 0 && !is.na(maxNumberOfSubjects) && 
 			maxNumberOfSubjects > 0 && allocationRatioPlanned == 0) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
@@ -1391,7 +1400,9 @@
 	}
 }
 
-.assertIsValidThetaH1 <- function(thetaH1, stageResults = NULL, stage = NULL, ..., results = NULL) {
+.assertIsValidThetaH1 <- function(thetaH1, stageResults = NULL, 
+		stage = NULL, ..., results = NULL) {
+		
 	if (is.na(thetaH1) && !is.null(stageResults) && !is.null(stage)) {
 		thetaH1 <- stageResults$effectSizes[stage]
 		if (!is.null(results)) {
@@ -1402,58 +1413,71 @@
 	invisible(thetaH1)
 }
 
-.assertIsValidAssumedStDev <- function(assumedStDev, stageResults = NULL, stage = NULL, ..., results = NULL) {
+.assertIsValidAssumedStDev <- function(assumedStDev, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+		
 	if (is.na(assumedStDev) && !is.null(stageResults) && !is.null(stage)) {
 		assumedStDev <- stageResults$overallStDevs[stage]
 		if (!is.null(results)) {
 			results$.setParameterType("assumedStDev", C_PARAM_GENERATED)
 		}
 	}
-
 	.assertIsSingleNumber(assumedStDev, "assumedStDev")
-	
 	if (assumedStDev <= 0) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT,
 			"'assumedStDev' (", assumedStDev, ") must be > 0")
 	}
-	
 	invisible(assumedStDev)
 }
 
-.assertIsValidThetaH1ForMultiArm <- function(thetaH1, stageResults = NULL, stage = NULL, ..., results = NULL) {
+.assertIsValidThetaH1ForMultiArm <- function(thetaH1, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+		
 	if (!is.null(stageResults) && all(is.na(thetaH1)) && !is.null(stage)) {
 		thetaH1 <- stageResults$effectSizes[, stage]
 		if (!is.null(results)) {
 			results$.setParameterType("thetaH1", C_PARAM_GENERATED)
 		}
 	}
-		
+
 	.assertIsNumericVector(thetaH1, "thetaH1", naAllowed = TRUE)
-	
 	invisible(thetaH1)
 }
 
-.assertIsValidAssumedStDevForMultiArm <- function(assumedStDev, stageResults = NULL, stage = NULL, ..., results = NULL) {
+.assertIsValidThetaH1ForEnrichment <- function(thetaH1, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+	invisible(.assertIsValidThetaH1ForMultiArm(thetaH1 = thetaH1, 
+		stageResults = stageResults, stage = stage, results = results))
+}
+
+.assertIsValidAssumedStDevForMultiHypotheses <- function(assumedStDev, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+		
 	if (!is.null(stageResults) && all(is.na(assumedStDev)) && !is.null(stage)) {
-		assumedStDev <- stageResults$overallStDevs[, stage]
+		
+		if (is.matrix(stageResults$overallStDevs)) { # inherits(stageResults, "StageResultsMultiArmMeans")
+			assumedStDev <- stageResults$overallStDevs[, stage]
+		} else {
+			assumedStDev <- stageResults$overallStDevs[stage]
+		}
+		
 		if (!is.null(results)) {
 			results$.setParameterType("assumedStDevs", C_PARAM_GENERATED)
 		}
 	}
-	
 	.assertIsNumericVector(assumedStDev, "assumedStDev", naAllowed = TRUE)
-	
 	if (any(assumedStDev <= 0, na.rm = TRUE)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT,
-				"'assumedStDev' (", .arrayToString(assumedStDev), ") must be > 0")
+			"'assumedStDev' (", .arrayToString(assumedStDev), ") must be > 0")
 	}
 	
 	invisible(assumedStDev)
 }
 
-.assertIsValidPiTreatmentsForMultiArm <- function(piTreatments, stageResults = NULL, stage = NULL, ..., results = NULL) {
+.assertIsValidPiTreatmentsForMultiArm <- function(piTreatments, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
 	if (!is.null(stageResults) && all(is.na(piTreatments)) && !is.null(stage)) {
-		piTreatments <- stageResults$piTreatments[, stage]
+		piTreatments <- stageResults$overallPiTreatments[, stage]
 		if (!is.null(results)) {
 			results$.setParameterType("piTreatments", C_PARAM_GENERATED)
 		}
@@ -1463,9 +1487,10 @@
 	invisible(piTreatments)
 }
 
-.assertIsValidPiControlForMultiArm <- function(piControl, stageResults = NULL, stage = NULL, ..., results = NULL) {
-	if (is.na(piControl) && !is.null(stageResults) && !is.null(stage)) {
-		piControl <- stageResults$piControl[,stage]
+.assertIsValidPiControlForMultiArm <- function(piControl, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+	if (!is.null(stageResults) && is.na(piControl) && !is.null(stage)) {
+		piControl <- stageResults$overallPiControl[, stage]
 		if (!is.null(results)) {
 			results$.setParameterType("piControl", C_PARAM_GENERATED)
 		}
@@ -1475,6 +1500,32 @@
 	invisible(piControl)
 }
 
+.assertIsValidPiTreatmentsForEnrichment <- function(piTreatments, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+	if (!is.null(stageResults) && all(is.na(piTreatments)) && !is.null(stage)) {
+		piTreatments <- stageResults$overallPisTreatment[, stage]
+		if (!is.null(results)) {
+			results$.setParameterType("piTreatments", C_PARAM_GENERATED)
+		}
+	}
+	.assertIsNumericVector(piTreatments, "piTreatments", naAllowed = TRUE)
+	.assertIsInClosedInterval(piTreatments, "piTreatments", lower = 0, upper = 1, naAllowed = TRUE)
+	invisible(piTreatments)
+}
+
+.assertIsValidPiControlForEnrichment <- function(piControls, 
+		stageResults = NULL, stage = NULL, ..., results = NULL) {
+	if (!is.null(stageResults) && all(is.na(piControls)) && !is.null(stage)) {
+		piControls <- stageResults$overallPisControl[, stage]
+		if (!is.null(results)) {
+			results$.setParameterType("piControls", C_PARAM_GENERATED)
+		}
+	}
+	.assertIsNumericVector(piControls, "piControls", naAllowed = TRUE)
+	.assertIsInClosedInterval(piControls, "piControls", lower = 0, upper = 1, naAllowed = TRUE)
+	invisible(piControls)
+}
+
 .isValidValueOrVector <- function(x) {
 	if (is.null(x) || length(x) == 0) {
 		return(FALSE)
@@ -1529,6 +1580,15 @@
 	return(directionUpper)
 }
 
+.assertIsFunction <- function(fun) {
+	if (is.null(fun)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'fun' must be a valid function")
+	}
+	if (!is.function(fun)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'fun' must be a function (is ", class(fun), ")")
+	}
+}
+
 .assertIsValidFunction <- function(fun, ..., 
 		funArgName = "fun", 
 		expectedArguments = NULL, 
@@ -1714,11 +1774,13 @@
 	}
 
 	param <- NA_character_
+	paramValues <- NA_real_
 	if (dataInput$isDatasetSurvival()) {
 		if (any(abs(design$informationRates[2:stage] - dataInput$getOverallEventsUpTo(stage)[2:stage] /
 				dataInput$getOverallEventsUpTo(1) * design$informationRates[1]) > 
 				C_ACCEPT_DEVIATION_INFORMATIONRATES)) {
 			param <- "events"
+			paramValues <- dataInput$getOverallEventsUpTo(stage)
 		}
 	} else {
 		if (dataInput$getNumberOfGroups() == 1) {	
@@ -1727,6 +1789,7 @@
 					dataInput$getOverallSampleSizesUpTo(1) * design$informationRates[1]) > 
 					C_ACCEPT_DEVIATION_INFORMATIONRATES)) {
 				param <- "sample sizes"
+				paramValues <- dataInput$getOverallSampleSizesUpTo(stage)
 			}
 		}
 		else if (dataInput$getNumberOfGroups() == 2) {	
@@ -1734,32 +1797,33 @@
 						dataInput$getOverallSampleSizesUpTo(stage)[2:stage] /
 						dataInput$getOverallSampleSizesUpTo(1) * design$informationRates[1]) > 
 						C_ACCEPT_DEVIATION_INFORMATIONRATES) ||
-					any(abs(design$informationRates[2:stage] - 
+				any(abs(design$informationRates[2:stage] - 
 						dataInput$getOverallSampleSizesUpTo(stage,2)[2:stage] / 
 						dataInput$getOverallSampleSizesUpTo(1, 2) * design$informationRates[1]) > 
 						C_ACCEPT_DEVIATION_INFORMATIONRATES)) {
 				param <- "sample sizes"
+				paramValues <- dataInput$getOverallSampleSizesUpTo(stage) + dataInput$getOverallSampleSizesUpTo(stage, 2)
 			}
 		}
 	}
 	if (!is.na(param)) {
-		warning("Observed ", param, " not according to specified information rates in ", 
+		warning("Observed ", param, " (", .arrayToString(paramValues), 
+			") not according to specified information rates (", 
+			.arrayToString(design$informationRates[1:stage]), ") in ", 
 			"group sequential design. ", 
 			"Test procedure might not control Type I error rate", call. = FALSE)
 	}
 }
 
-.assertIsOneSidedForMultiArmAnalysis <- function(design, dataInput) {
-	if (design$sided == 2 && .isMultiArmDataset(dataInput)) {
-		stop(C_EXCEPTION_TYPE_CONFLICTING_ARGUMENTS, 
-			"multi-arm analysis is only applicable for one-sided testing")
-	}
-}
-
-.assertIsOneSidedDesign <- function(design) {
+.assertIsOneSidedDesign <- function(design, 
+		designType = c("multi-arm", "enrichment"), 
+		engineType = c("simulation", "analysis")) {
+		
 	if (design$sided == 2) {
+		designType <- match.arg(designType)
+		engineType <- match.arg(engineType)
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
-			"multi-arm simulation is only applicable for one-sided testing")
+			designType, " ", engineType, " is only applicable for one-sided testing")
 	}
 }
 
@@ -1768,17 +1832,46 @@
 }
 
 .isMultiArmStageResults <- function(stageResults) {
-	return(grepl("MultiArm", class(stageResults)))
+	return(inherits(stageResults, "StageResults") && grepl("MultiArm", class(stageResults)))
+}
+
+.isEnrichmentStageResults <- function(stageResults) {
+	return(inherits(stageResults, "StageResults") && grepl("Enrichment", class(stageResults)))
+}
+
+.isEnrichmentConditionalPowerResults <- function(conditionalPowerResults) {
+	return(inherits(conditionalPowerResults, "ConditionalPowerResults") && 
+		grepl("Enrichment", class(conditionalPowerResults)))
+}
+
+.isMultiHypothesesStageResults <- function(x) {
+	return(.isMultiArmStageResults(x) || .isEnrichmentStageResults(x))
 }
 
 .isMultiArmAnalysisResults <- function(analysisResults) {
 	return(inherits(analysisResults, "AnalysisResultsMultiArm"))
 }
 
+.isMultiHypothesesAnalysisResults <- function(x) {
+	return(.isMultiArmAnalysisResults(x) || .isEnrichmentAnalysisResults(x))
+}
+
+.isEnrichmentDataset <- function(dataInput) {
+	return(inherits(dataInput, "Dataset") && dataInput$.enrichmentEnabled)
+}
+
+.isEnrichmentAnalysisResults <- function(analysisResults) {
+	return(inherits(analysisResults, "AnalysisResultsEnrichment"))
+}
+
 .isMultiArmSimulationResults <- function(simulationResults) {
 	return(inherits(simulationResults, "SimulationResults") && grepl("MultiArm", class(simulationResults)))
 }
 
+.isEnrichmentSimulationResults <- function(simulationResults) {
+	return(inherits(simulationResults, "SimulationResults") && grepl("Enrichment", class(simulationResults)))
+}
+
 .assertIsStageResultsMultiArm <- function(stageResults) {
 	if (!inherits(stageResults, "StageResults")) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
@@ -1791,12 +1884,17 @@
 	}
 }
 
-.assertIsStageResultsNonMultiArm <- function(stageResults) {
+.assertIsStageResultsNonMultiHypotheses <- function(stageResults) {
 	if (inherits(stageResults, "StageResults") && .isMultiArmStageResults(stageResults)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
 			"'stageResults' must be a non-multi-arm object (is ", class(stageResults), ")")
 	}
 	
+	if (inherits(stageResults, "StageResults") && .isEnrichmentStageResults(stageResults)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
+			"'stageResults' must be a non-enrichment object (is ", class(stageResults), ")")
+	}
+	
 	allowedClasses <- c(
 		"StageResultsMeans", 
 		"StageResultsRates",
@@ -1808,11 +1906,16 @@
 	}
 }
 
-.assertIsDatasetNonMultiArm <- function(dataInput) {
+.assertIsDatasetNonMultiHypotheses <- function(dataInput) {
 	if (.isMultiArmDataset(dataInput)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
 			"'dataInput' must be a non-multi-arm dataset (has ", dataInput$getNumberOfGroups(), " treatment arms)")
 	}
+	if (.isEnrichmentDataset(dataInput)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
+				"'dataInput' must be a non-enrichment dataset (has ", dataInput$getNumberOfSubsets(), " subsets)")
+	}
+	
 }
 
 .assertIsAnalysisResults <- function(analysisResults) {
@@ -1822,22 +1925,22 @@
 	}
 }
 
-.isValidIntersectionTest <- function(intersectionTest) {
+.isValidIntersectionTestMultiArm <- function(intersectionTest) {
 	return(!is.null(intersectionTest) && length(intersectionTest) == 1 && !is.na(intersectionTest) && 
-		is.character(intersectionTest) && intersectionTest %in% C_INTERSECTION_TESTS)
+		is.character(intersectionTest) && intersectionTest %in% C_INTERSECTION_TESTS_MULTIARMED)
 }
 
-.getCorrectedIntersectionTestIfNecessary <- function(design, intersectionTest, userFunctionCallEnabled = TRUE) {
+.getCorrectedIntersectionTestMultiArmIfNecessary <- function(design, intersectionTest, userFunctionCallEnabled = TRUE) {
 	.assertIsCharacter(intersectionTest, "intersectionTest")
 	intersectionTest <- intersectionTest[1]
 	if (.isTrialDesignConditionalDunnett(design) && intersectionTest != "Dunnett") {
 		if (userFunctionCallEnabled) {
 			message <- paste0("Intersection test '", intersectionTest, "' ")
-			if (!.isValidIntersectionTest(intersectionTest)) {
+			if (!.isValidIntersectionTestMultiArm(intersectionTest)) {
 				message <- paste0(message, "is invalid, ")
 			}
 			message <- paste0(message, "will be ignored")
-			message <- paste0(message, ifelse(!.isValidIntersectionTest(intersectionTest), ", ", " "))
+			message <- paste0(message, ifelse(!.isValidIntersectionTestMultiArm(intersectionTest), ", ", " "))
 			message <- paste0(message, "and 'Dunnett' will be used instead ",
 				"because conditional Dunnett test was specified as design")
 			warning(message, call. = FALSE)
@@ -1847,19 +1950,34 @@
 	return(intersectionTest)
 }
 
-.assertIsValidIntersectionTest <- function(design, intersectionTest) {
+.assertIsValidIntersectionTestMultiArm <- function(design, intersectionTest) {
 	.assertIsCharacter(intersectionTest, "intersectionTest")
 	intersectionTest <- intersectionTest[1]
-	if (!.isValidIntersectionTest(intersectionTest)) {
+	if (!.isValidIntersectionTestMultiArm(intersectionTest)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'intersectionTest' (", intersectionTest, ") must be one of ",
-			.arrayToString(C_INTERSECTION_TESTS, encapsulate = TRUE))
+			.arrayToString(C_INTERSECTION_TESTS_MULTIARMED, encapsulate = TRUE))
 	}
-	if (.isTrialDesignConditionalDunnett(design) && intersectionTest != C_INTERSECTION_TEST_DUNNETT) {
+	if (.isTrialDesignConditionalDunnett(design) && intersectionTest != "Dunnett") {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "intersection test ('", intersectionTest, "') must be 'Dunnett' ",
 			"because conditional Dunnett test was specified as design")
 	}
 }
 
+.isValidIntersectionTestEnrichment <- function(intersectionTest) {
+	return(!is.null(intersectionTest) && length(intersectionTest) == 1 && !is.na(intersectionTest) && 
+					is.character(intersectionTest) && intersectionTest %in% C_INTERSECTION_TESTS_ENRICHMENT)
+}
+
+.assertIsValidIntersectionTestEnrichment <- function(design, intersectionTest) {
+	.assertIsCharacter(intersectionTest, "intersectionTest")
+	intersectionTest <- intersectionTest[1]
+	if (!.isValidIntersectionTestEnrichment(intersectionTest)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'intersectionTest' (", intersectionTest, ") must be one of ",
+				.arrayToString(C_INTERSECTION_TESTS_ENRICHMENT, encapsulate = TRUE))
+	}
+	return(intersectionTest)
+}
+
 .ignoreParameterIfNotUsed <- function(paramName, paramValue, requirementLogical, requirementFailedReason,
 		prefix = NA_character_) {
 	if (all(is.na(paramValue)) || requirementLogical) {
@@ -1914,15 +2032,15 @@
 	}
 }
 
-.isValidVarianceOption <- function(varianceOption) {
+.isValidVarianceOptionMultiArmed <- function(varianceOption) {
 	return(!is.null(varianceOption) && length(varianceOption) == 1 && !is.na(varianceOption) && 
-			is.character(varianceOption) && varianceOption %in% C_VARIANCE_OPTIONS)
+			is.character(varianceOption) && varianceOption %in% C_VARIANCE_OPTIONS_MULTIARMED)
 }
 
-.assertIsValidVarianceOption <- function(design, varianceOption) {
-	if (!.isValidVarianceOption(varianceOption)) {
+.assertIsValidVarianceOptionMultiArmed <- function(design, varianceOption) {
+	if (!.isValidVarianceOptionMultiArmed(varianceOption)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'varianceOption' should be one of ",
-			.arrayToString(C_VARIANCE_OPTIONS, encapsulate = TRUE))
+			.arrayToString(C_VARIANCE_OPTIONS_MULTIARMED, encapsulate = TRUE))
 	}
 	if (.isTrialDesignConditionalDunnett(design) && varianceOption != C_VARIANCE_OPTION_DUNNETT) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, 
@@ -1931,6 +2049,21 @@
 	}
 }
 
+
+.isValidVarianceOptionEnrichment <- function(varianceOption) {
+	return(!is.null(varianceOption) && length(varianceOption) == 1 && !is.na(varianceOption) && 
+					is.character(varianceOption) && varianceOption %in% C_VARIANCE_OPTIONS_ENRICHMENT)
+}
+
+
+.assertIsValidVarianceOptionEnrichment <- function(design, varianceOption) {
+	if (!.isValidVarianceOptionEnrichment(varianceOption)) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'varianceOption' should be one of ",
+				.arrayToString(C_VARIANCE_OPTIONS_ENRICHMENT, encapsulate = TRUE))
+	}
+}
+
+
 .assertIsValidSummaryIntervalFormat <- function(intervalFormat) {
 	.assertIsSingleCharacter(intervalFormat, "intervalFormat") # "[%s; %s]"
 	if (!grepl("^[^%]*%s[^%]*%s[^%]*$",  intervalFormat)) {
@@ -1998,11 +2131,27 @@
 	return(effectMeasure)
 }
 
-.assertIsValidMatrix <- function(x, argumentName, naAllowed = FALSE) {
+.assertIsValidMatrix <- function(x, argumentName, ..., 
+		expectedNumberOfColumns = NA_integer_, naAllowed = FALSE, returnSingleValueAsMatrix = FALSE) {
+		
 	if (missing(x) || is.null(x) || length(x) == 0) {
 		stop(C_EXCEPTION_TYPE_MISSING_ARGUMENT, "'", argumentName, "' must be a valid matrix")
 	}
 	
+	if (returnSingleValueAsMatrix && !is.matrix(x) && (is.numeric(x) || is.character(x) || is.logical(x))) {
+		if (length(x) == 1) {
+			x <- matrix(x)
+		}
+		else if (length(x) > 1 && !is.na(expectedNumberOfColumns)) {
+			if (length(x) %% expectedNumberOfColumns != 0) {
+				stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "the length of '", argumentName, "' (", .arrayToString(x), 
+					") must be a divisor or a multiple ", expectedNumberOfColumns)
+			}
+			
+			x <- matrix(x, ncol = expectedNumberOfColumns)
+		}
+	}
+	
 	if (!is.matrix(x)) {
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'", argumentName, "' (", class(x), ") must be a valid matrix")
 	}
@@ -2015,8 +2164,16 @@
 		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'", argumentName, "' (", 
 			.arrayToString(x), ") must be a valid numeric matrix")
 	}
+	
+	if (!is.na(expectedNumberOfColumns) && ncol(x) != expectedNumberOfColumns) {
+		stop(C_EXCEPTION_TYPE_ILLEGAL_ARGUMENT, "'", argumentName, "' (", 
+			.arrayToString(x), ") must be a numeric matrix with ", expectedNumberOfColumns, " columns")
+	}
+	
+	return(invisible(x))
 }
 
+
 .assertIsValidTypeOfShape <- function(typeOfShape) {
 	.assertIsCharacter(typeOfShape, "typeOfShape")
 	typeOfShape <- typeOfShape[1]
@@ -2030,7 +2187,8 @@
 .assertIsValidEffectMatrixMeans <- function(typeOfShape, effectMatrix, muMaxVector, gED50, gMax, slope) {
 	
 	if (typeOfShape == "userDefined") {
-		.assertIsValidMatrix(effectMatrix, "effectMatrix", naAllowed = FALSE)
+		effectMatrix <- .assertIsValidMatrix(effectMatrix, "effectMatrix", 
+			expectedNumberOfColumns = gMax, naAllowed = FALSE, returnSingleValueAsMatrix = TRUE)
 		
 		.assertIsNumericVector(muMaxVector, "muMaxVector", naAllowed = TRUE)
 		if (!all(is.na(muMaxVector)) && !identical(muMaxVector, C_ALTERNATIVE_POWER_SIMULATION_DEFAULT)) {
@@ -2069,7 +2227,8 @@
 .assertIsValidEffectMatrixRates <- function(typeOfShape, effectMatrix, piMaxVector, piControl, gED50, gMax, slope) {
 	
 	if (typeOfShape == "userDefined") {
-		.assertIsValidMatrix(effectMatrix, "effectMatrix", naAllowed = FALSE)
+		effectMatrix <- .assertIsValidMatrix(effectMatrix, "effectMatrix", 
+			expectedNumberOfColumns = gMax, naAllowed = FALSE, returnSingleValueAsMatrix = TRUE)
 		.assertIsInOpenInterval(effectMatrix, "effectMatrix", 0, 1, naAllowed = FALSE)
 		
 		.assertIsNumericVector(piMaxVector, "piMaxVector", naAllowed = TRUE)
@@ -2114,7 +2273,8 @@
 .assertIsValidEffectMatrixSurvival <- function(typeOfShape, effectMatrix, omegaMaxVector, gED50, gMax, slope) {
 	
 	if (typeOfShape == "userDefined") {
-		.assertIsValidMatrix(effectMatrix, "effectMatrix", naAllowed = FALSE)
+		effectMatrix <- .assertIsValidMatrix(effectMatrix, "effectMatrix", 
+			expectedNumberOfColumns = gMax, naAllowed = FALSE, returnSingleValueAsMatrix = TRUE)
 		.assertIsInOpenInterval(effectMatrix, "effectMatrix", 0, NULL, naAllowed = FALSE)
 		
 		.assertIsNumericVector(omegaMaxVector, "omegaMaxVector", naAllowed = TRUE)
@@ -2152,7 +2312,7 @@
 	}
 	
 	return(effectMatrix)
-}
+}	
 
 .assertIsValidPlannedSubjects <- function(plannedSubjects, kMax) {
 	
@@ -2166,3 +2326,11 @@
 	.assertValuesAreStrictlyIncreasing(plannedSubjects, "plannedSubjects")
 }
 
+.isAlphaSpendingDesign <- function(design) {
+	if (!.isTrialDesignInverseNormalOrGroupSequential(design)) {
+		return(FALSE)
+	}
+	
+	return(grepl("^as", design$typeOfDesign))
+}
+
